@@ -33,20 +33,24 @@
 
 #ifndef	lint
 static char *moduleid = 
-	"@(#)$Id: apprentice.c,v 1.19 1994/05/03 17:58:23 christos Exp $";
+	"@(#)$Id: apprentice.c,v 1.22 1995/05/20 22:09:21 christos Exp $";
 #endif	/* lint */
 
 #define	EATAB {while (isascii((unsigned char) *l) && \
 		      isspace((unsigned char) *l))  ++l;}
+#define LOWCASE(l) (isupper((unsigned char) (l)) ? \
+			tolower((unsigned char) (l)) : (l))
 
 
 static int getvalue	__P((struct magic *, char **));
 static int hextoint	__P((int));
 static char *getstr	__P((char *, char *, int, int *));
 static int parse	__P((char *, int *, int));
+static void eatsize	__P((char **));
 
 static int maxmagic = 0;
 
+
 int
 apprentice(fn, check)
 char *fn;			/* name of magic file */
@@ -147,8 +151,9 @@ int *ndx, check;
 	struct magic *m;
 	char *t, *s;
 
+#define ALLOC_INCR	20
 	if (nd+1 >= maxmagic){
-	    maxmagic += 20;
+	    maxmagic += ALLOC_INCR;
 	    if ((magic = (struct magic *) realloc(magic, 
 						  sizeof(struct magic) * 
 						  maxmagic)) == NULL) {
@@ -158,6 +163,7 @@ int *ndx, check;
 		else
 			exit(1);
 	    }
+	    memset(&magic[*ndx], 0, sizeof(struct magic) * ALLOC_INCR);
 	}
 	m = &magic[*ndx];
 	m->flag = 0;
@@ -174,7 +180,7 @@ int *ndx, check;
 	}
 
 	/* get offset, then skip over it */
-	m->offset = (int) strtol(l,&t,0);
+	m->offset = (int) strtoul(l,&t,0);
         if (l == t)
 		magwarn("offset %s invalid", l);
         l = t;
@@ -186,13 +192,16 @@ int *ndx, check;
 		 * read [.lbs][+-]nnnnn)
 		 */
 		if (*l == '.') {
-			switch (*++l) {
+			l++;
+			switch (LOWCASE(*l)) {
 			case 'l':
 				m->in.type = LONG;
 				break;
+			case 'h':
 			case 's':
 				m->in.type = SHORT;
 				break;
+			case 'c':
 			case 'b':
 				m->in.type = BYTE;
 				break;
@@ -205,7 +214,7 @@ int *ndx, check;
 		s = l;
 		if (*l == '+' || *l == '-') l++;
 		if (isdigit((unsigned char)*l)) {
-			m->in.offset = strtol(l, &t, 0);
+			m->in.offset = strtoul(l, &t, 0);
 			if (*s == '-') m->in.offset = - m->in.offset;
 		}
 		else
@@ -278,7 +287,8 @@ int *ndx, check;
 	/* New-style anding: "0 byte&0x80 =0x80 dynamically linked" */
 	if (*l == '&') {
 		++l;
-		m->mask = signextend(m, strtol(l, &l, 0));
+		m->mask = signextend(m, strtoul(l, &l, 0));
+		eatsize(&l);
 	} else
 		m->mask = ~0L;
 	EATAB;
@@ -360,8 +370,10 @@ char **p;
 		*p = getstr(*p, m->value.s, sizeof(m->value.s), &slen);
 		m->vallen = slen;
 	} else
-		if (m->reln != 'x')
-			m->value.l = signextend(m, strtol(*p, p, 0));
+		if (m->reln != 'x') {
+			m->value.l = signextend(m, strtoul(*p, p, 0));
+			eatsize(p);
+		}
 	return 0;
 }
 
@@ -549,3 +561,30 @@ int len;
 		}
 	}
 }
+
+/*
+ * eatsize(): Eat the size spec from a number [eg. 10UL]
+ */
+static void
+eatsize(p)
+char **p;
+{
+	char *l = *p;
+
+	if (LOWCASE(*l) == 'u') 
+		l++;
+
+	switch (LOWCASE(*l)) {
+	case 'l':    /* long */
+	case 's':    /* short */
+	case 'h':    /* short */
+	case 'b':    /* char/byte */
+	case 'c':    /* char/byte */
+		l++;
+		/*FALLTHROUGH*/
+	default:
+		break;
+	}
+
+	*p = l;
+}
